###################################################################################################
# _textformatmanager.py
#
# Implementation of class TextFormatManager (see below).
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
###################################################################################################

# Imports.
from __future__ import nested_scopes
from Globals import HTMLFile
import copy
import urllib
# Product Imports.
import _globals 


# -------------------------------------------------------------------------------------------------
#  importXml
# -------------------------------------------------------------------------------------------------

def _importXml(self, item):
  textFmts = getTextFormats(self)
  id = item['key']
  dict = item['value']
  print "[_textformatmanager._importXml]: %s"%(id)
  if id in textFmts:
    i = textFmts.index(id)
    textFmts[i+1] = dict
  else:
    textFmts.append(id)
    textFmts.append(dict)
  self.setConfProperty('ZMS.custom.textformats',copy.deepcopy(textFmts))

def importXml(self, xml):
  value = self.parseXmlString(xml)
  if type(value) is type([]):
    for item in value:
      _importXml(self,item)
  else:
    _importXml(self,value)


# ------------------------------------------------------------------------------------------
#  _textformatmanager.br_quote
# ------------------------------------------------------------------------------------------
def br_quote(text, subtag, REQUEST):
  rtn = ''
  q = ''
  if _globals.isManagementInterface(REQUEST):
    if not REQUEST.has_key('format'):
      q = ''
  if subtag == 'br':
    tmp = []
    for s in text.split('<%s>'%subtag):
      while len(s) > 0 and ord(s[0]) in [10,13]: s = s[1:]
      if len(tmp) > 0:
        tmp.append('\n')
      tmp.append(s)
    text = "".join(tmp)
  for s in text.split('\n'):
    subtext = s.strip()
    if subtag == 'br':
      rtn = rtn + '\n%s%s<%s>'%(subtext,q,subtag)
    elif len(subtext) > 0:
      rtn = rtn + '\n<%s>%s%s</%s>'%(subtag,subtext,q,subtag)
  return rtn


# ------------------------------------------------------------------------------------------
#  _textformatmanager.delTextFormat:
# ------------------------------------------------------------------------------------------
def delTextFormat(self, id):
  textFmts = getTextFormats(self)
  i = textFmts.index(id)
  del textFmts[i]
  del textFmts[i]
  self.setConfProperty('ZMS.custom.textformats',copy.deepcopy(textFmts))


# ------------------------------------------------------------------------------------------
#  _textformatmanager.setTextFormat:
# ------------------------------------------------------------------------------------------
def setTextFormat(self, id, newId, newDisplay, manage_lang, newTag='', newSubtag='', newAttrs='', newRichedit=0):
  textFmts = getTextFormats(self)
  if id in textFmts:
    i = textFmts.index(id)
  else:
    i = len(textFmts)
    textFmts.append(newId)
    textFmts.append({'display':{}})
  dict = textFmts[i+1]
  dict['display'][manage_lang] = newDisplay
  dict['tag'] = newTag
  dict['subtag'] = newSubtag
  dict['attrs'] = newAttrs
  dict['richedit'] = newRichedit
  textFmts[i] = newId
  textFmts[i+1] = dict.copy()
  self.setConfProperty('ZMS.custom.textformats',copy.deepcopy(textFmts))


# ------------------------------------------------------------------------------------------
#  _textformatmanager.setDefaultTextFormat:
# ------------------------------------------------------------------------------------------
def setDefaultTextFormat(self, id):
  if len(id) > 0:
    self.setConfProperty('ZMS.custom.textformats.default',id)


# ------------------------------------------------------------------------------------------
#  _textformatmanager.getTextFormats:
# ------------------------------------------------------------------------------------------
def getTextFormats(self):
  return self.getConfProperty('ZMS.custom.textformats',[])


###################################################################################################
###################################################################################################
###
###   C l a s s   T e x t F o r m a t O b j e c t:
###
###################################################################################################
###################################################################################################
class TextFormatObject:

    # ------------------------------------------------------------------------------------------
    #  TextFormatObject.getTextFormat:
    # ------------------------------------------------------------------------------------------
    def getTextFormat(self, format, REQUEST):
      obs = getTextFormats(self.getDocumentElement())
      for i in range(0,len(obs)/2):
        id = obs[i*2] 
        ob = obs[i*2+1]
        if id == format: 
          return TextFormat(self,id,ob,REQUEST)
      return None

    # ------------------------------------------------------------------------------------------
    #  TextFormatObject.renderText:
    # ------------------------------------------------------------------------------------------
    def renderText(self, id, format, text, REQUEST):
      # Process format.
      textformat = self.getTextFormat(format,REQUEST)
      if textformat is not None:
        text = textformat.renderText(id,text,REQUEST)
      # Process html <form>-tags.
      text = _globals.form_quote(text,REQUEST)
      # Return.
      return text

    # ------------------------------------------------------------------------------------------
    #  TextFormatObject.renderPrint:
    # ------------------------------------------------------------------------------------------
    def renderPrint(self, pre, format, text, REQUEST):
      # Process format.
      textformat = self.getTextFormat(format,REQUEST)
      if textformat is not None:
        text = textformat.renderPrint(pre,text,REQUEST)
      # Process html <form>-tags.
      text = _globals.form_quote(text,REQUEST)
      # Return.
      return text


###################################################################################################
###################################################################################################
###
###   C l a s s   T e x t F o r m a t M a n a g e r :
###
###################################################################################################
###################################################################################################
class TextFormatManager: 

    # Management Permissions.
    # -----------------------
    __authorPermissions__ = ()
    __editorPermissions__ = ()
    __administratorPermissions__ = (
                'manage_customizeTextFormatForm','manage_customizeTextFormat',
                )
    __ac_permissions__=(
                ('ZMS Administrator', __administratorPermissions__),
                ('ZMS Author', __authorPermissions__),
                ('ZMS Editor', __editorPermissions__),
                )


    # Management Interface.
    # ---------------------
    manage_customizeTextFormatForm = HTMLFile('dtml/textformatmanager/manage_customizeform', globals()) 


    # ------------------------------------------------------------------------------------------
    #  TextFormatManager.getTextFormatIds:
    # ------------------------------------------------------------------------------------------
    def getTextFormatIds(self, REQUEST):
      return map(lambda ob: ob.getId(),self.getTextFormats(REQUEST))

    # ------------------------------------------------------------------------------------------
    #  TextFormatManager.getTextFormats:
    # ------------------------------------------------------------------------------------------
    def getTextFormats(self, REQUEST):
      l_sort = []
      obs = getTextFormats(self.getDocumentElement())
      for i in range(0,len(obs)/2):
        id = obs[i*2+0] 
        ob = obs[i*2+1]
        textformat = TextFormat(self,id,ob,REQUEST)
        l_sort.append((textformat.getDisplay(),textformat))
      l_sort.sort()
      return map(lambda ob: ob[1],l_sort)

    # ------------------------------------------------------------------------------------------
    #  TextFormatManager.getTextFormatDefault:
    # ------------------------------------------------------------------------------------------
    def getTextFormatDefault(self, REQUEST={}):
      return self.getConfProperty('ZMS.custom.textformats.default','body')


    ############################################################################################
    # TextFormatManager.manage_customizeTextFormat:
    #
    # Einstellungen fr Absatzformate ndern.
    ############################################################################################
    def manage_customizeTextFormat(self, lang, manage_lang, REQUEST, RESPONSE): 
      """ TextFormatManager.manage_customizeTextFormat """

      message = ''
      id = REQUEST.get('id','')
      
      # Change.
      # -------
      if REQUEST['btn'] == self.getLangStr('BTN_CHANGE',manage_lang):
        old_id = REQUEST['id']
        id = REQUEST['new_id'].strip()
        display = REQUEST['new_display'].strip()
        tag = REQUEST['new_tag'].strip()
        subtag = REQUEST['new_subtag'].strip()
        attrs = REQUEST['new_attrs'].strip()
        richedit = 0
        if REQUEST.has_key('new_richedit'):
          richedit = 1
        setTextFormat(self,old_id,id,display,manage_lang,tag,subtag,attrs,richedit)
        if REQUEST.has_key('new_default'): 
          setDefaultTextFormat(self,REQUEST['new_default'])
        message = self.getLangStr('MSG_CHANGED',manage_lang)
      
      # Delete.
      # -------
      elif REQUEST['btn'] == self.getLangStr('BTN_DELETE',manage_lang):
        id = REQUEST['id']
        delTextFormat(self,id) 
        id = ""
        message = self.getLangStr('MSG_DELETED',manage_lang)%int(1)
      
      # Insert.
      # -------
      elif REQUEST['btn'] == self.getLangStr('BTN_INSERT',manage_lang):
        id = REQUEST['_id'].strip()
        display = REQUEST['_display'].strip()
        setTextFormat(self,id,display,manage_lang)
        message = self.getLangStr('MSG_CHANGED',manage_lang)
      
      # Export.
      # -------
      elif REQUEST['btn'] == self.getLangStr('BTN_EXPORT',manage_lang):
        value = []
        ids = REQUEST.get('ids',[])
        textFmts = getTextFormats(self)
        for i in range(0,len(textFmts)/2):
          id = textFmts[i*2]
          ob = textFmts[i*2+1]
          if id in ids or len(ids) == 0:
            value.append({'key':id,'value':ob})
        if len(value)==1:
          value = value[0]
        content_type = 'text/xml'
        filename = 'textFmt.xml'
        export = '<?xml version="1.0" encoding="ISO-8859-1"?>'
        export = export + self.toXmlString(value,1)
        RESPONSE.setHeader('Content-Type',content_type)
        RESPONSE.setHeader('Content-Disposition','inline;filename=%s'%filename)
        return export

      # Import.
      # -------
      elif REQUEST['btn'] == self.getLangStr('BTN_IMPORT',manage_lang):
        importXml(self,xml=REQUEST['file'])
        message = self.getLangStr('MSG_IMPORTED',manage_lang)%('<i>%s</i>'%REQUEST['file'].filename)

      # Return with message.
      message = urllib.quote(message)
      return RESPONSE.redirect('manage_customizeTextFormatForm?lang=%s&manage_lang=%s&manage_tabs_message=%s&id=%s'%(lang,manage_lang,message,id))


###################################################################################################
#  C L A S S   T E X T F O R M A T
###################################################################################################
class TextFormat:

  # Constructor.
  # ------------
  def __init__(self, owner, id, ob, REQUEST):
    #++ _globals.writeLog("[TextFormat.__init__]: id=%s; ob=%s"%(str(id),str(ob)))
    self.setOwner(owner)
    self.setId(id)
    if REQUEST is not None and \
       REQUEST.has_key('manage_lang') and \
       ob['display'].has_key(REQUEST['manage_lang']):
      self.setDisplay(ob['display'][REQUEST['manage_lang']])
    else:
      self.setDisplay(id)
    self.setTag(ob['tag'])
    self.setSubTag(ob['subtag'])
    self.setAttrs(ob['attrs'])
    richedit = 0
    if ob.has_key('richedit'):
      richedit = ob['richedit']
    self.setRichedit(richedit)
    

  # Owner
  # -----
  def getOwner(self): return self.owner
  def setOwner(self, owner): self.owner = owner

  # ID
  # --
  def getId(self): return self.id
  def setId(self, id): self.id = id

  # Display
  # -------
  def getDisplay(self): return self.display
  def setDisplay(self, display): self.display = display

  # <Tag>
  # -----
  def getTag(self): return self.tag
  def setTag(self, tag): self.tag = tag

  # <Sub-Tag>
  # -----------
  def getSubTag(self): return self.subtag
  def setSubTag(self, subtag): self.subtag = subtag

  # <Tag>-Attributes
  # ----------------
  def getAttrs(self): return self.attrs
  def setAttrs(self, attrs): self.attrs = attrs

  # Richedit
  # ----------------
  def getRichedit(self): return self.richedit
  def setRichedit(self, richedit): self.richedit = richedit

  # HTML
  # ----
  def getHtml(self): 
    html = ""
    # Open tag.
    if len(self.getTag()) > 0:
      html = html + "&lt;"
      html = html + self.getTag()
      if len(self.getAttrs()) > 0:
        html = html + " " + self.getAttrs()
      html = html + "&gt;"
      html = html + "<br>"
    # Sub tag.
    subtag = self.getSubTag()
    if len(subtag)>0:
      if subtag == 'br':
        html = html + "&nbsp;&nbsp;...&lt;" + subtag + "&gt;<br>"
      else:
        html = html + "&nbsp;&nbsp;&lt;" + subtag + "&gt;...&lt;/" + subtag + "&gt;<br>"
    else:
      html = html + "..."
    # Close tag.
    if len(self.getTag()) > 0:
      html = html + "&lt;/"
      html = html + self.getTag()
      html = html + "&gt;"
      html = html + "<br>"
    # Default.
    if len(html)==0:
      html = html + "&nbsp;"
    return html

  # Render text
  # -----------
  def renderText(self, id, text, REQUEST):
    html = ''
    # Open tag.
    tag = self.getTag()
    if len(tag) > 0:
      html = '%s<%s'%(html,tag)
      if id is not None:
        html = '%s id="%s"'%(html,id)
      if len(self.getAttrs()) > 0:
        html = '%s %s'%(html,self.getAttrs())
      html = '%s>'%(html)
    # Sub tag.
    subtag = self.getSubTag()
    if len(subtag) > 0:
      text = br_quote(text,subtag,REQUEST)
    html='%s%s'%(html,text)
    # Close tag.
    if len(tag) > 0:
      html = '%s</%s>'%(html,tag)
    # Default.
    if len(html)==0:
      html = '%s&nbsp;'%(html)
    # Return.
    return html

  # Render print
  # ------------
  def renderPrint(self, pre, text, REQUEST):
    html = ""
    # Open tag.
    tag = self.getTag()
    if len(tag) > 0:
      if tag == 'h1':
        html = html + '<p class="standard"><font size="4" style="font-weight:bold">'
      elif tag == 'h2':
        html = html + '<p class="standard"><font size="4" style="font-weight:bold; font-style:italic;">'
      elif tag == 'h3':
        html = html + '<p class="standard"><font size="3" style="font-weight:bold">'
      elif tag == 'h4':
        html = html + '<p class="standard"><font size="3" style="font-weight:bold; font-style:italic;">'
      elif tag == 'h5':
        html = html + '<p class="standard"><font size="2" style="font-weight:bold">'
      elif tag == 'h6':
        html = html + '<p class="standard"><font size="2" style="font-weight:bold; font-style:italic;">'
      else:
        html = html + "<%s"%tag
        if len(self.getAttrs()) > 0:
          html = html + " " + self.getAttrs()
        html = html + ">"
    # Pre.
    html = html + pre
    # Sub tag.
    subtag = self.getSubTag()
    if len(subtag) == 0:
      html = html + text
    else:
      html = html + br_quote(text,subtag,REQUEST)
    # Close tag.
    if len(tag) > 0:
      if tag in ['h1','h2','h3','h4','h5','h6']:
        html = html + "</font></p>"
      else:
        html = html + "</%s>"%tag
    # Default.
    if len(html)==0:
      html = html + "&nbsp;"
    # Return.
    return html

###################################################################################################
